/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Update 20/03/01 trace of the loaded libraries PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.kernel.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL

/*!
\ingroup Component
Context parameter for the component server.
Only the inprocess servers are implemented.
See DCOM for more informations.
*/
const t_ulong INPROC_SERVER=1;


const char* g_NAME_HYPDLLGETCLASSOBJECT="hypDllGetClassObject";
const char* g_NAME_HYPDLLCANUNLOADNOW="hypDllCanUnloadNow";


typedef std::set<std::string> t_ContDllLoaded;
t_ContDllLoaded gs_DllLoaded;



/*!
\ingroup Component
Gets the full path of the inprocess server.
\param Guid CLSID of the component which is loacated in the inprocess server.
\param PathInprocess Path of the server that is filled by the function
\return Result code. Test with FAILED to check if the result code is a error code.
*/
hyp_ker_DLL com_result hypGetPathInprocessServer(const CGuid& Guid, std::string& PathInprocess)
{
	try{
		CRegistryUtility reg_utility;
		PathInprocess=reg_utility.GetPathInprocessServer(Guid);
	} catch (CException&) {
		return com_false;
	}
	return com_ok;
}

/*!
\ingroup Component
Gets the pointer to a ClassFactory interface. This pointer is used to create components.
\param clsid CLSID of the component that will be created with the use of the interface.
\param ClsContext Server context. Only INPROC_SERVER is accepted
\param pServerInfo Information about the server. Must be equal to 0
\param iid IID of the ClassFactory interface. Usually, the interface is IClassFactory.
\param ppv Pointer of the ClassFactory interface.
\return Result code.
*/
hyp_ker_DLL com_result GetClassObject(const hypCLSID& clsid,
				  t_ulong ClsContext,
				  hypSERVERINFO* pServerInfo,
				  const hypIID& iid,
				  void** ppv)
{
	if(ClsContext==INPROC_SERVER && pServerInfo==0)
	{
		std::string path_composant;
		if(Failed( hypGetPathInprocessServer(clsid,path_composant) )) 
			return com_false;

		CDllUtility dll;
		bool check=false;
		try {
			dll.LoadLibrary(path_composant,true);
			
			check=true;
			PTR_HYPDLLGETCLASSOBJECT GetClassObject=static_cast<PTR_HYPDLLGETCLASSOBJECT>(
				dll.GetFunctionAddress(path_composant,g_NAME_HYPDLLGETCLASSOBJECT));

			gs_DllLoaded.insert(path_composant);
			return GetClassObject(clsid,iid,ppv);

		} catch(CException&) {
			if(check) dll.UnloadLibrary(path_composant);
			return com_false;			
		}

	} else {
		return com_false;
	}
}


/*!
\ingroup Component
Creates an component. The Component Server must implement the IClassFactory interface.
\param clsid CLSID of the created component.
\param pUnknownOuter Pointer to a component which is used for the aggregation.
\param ClsContext Server context. Only INPROC_SERVER is accepted
\param iid Selected IID of the created component.
\param ppv Pointer on the created component.
\return Result code. If the function fails :
- The return code is an error code
- ppv is set to 0
*/
hyp_ker_DLL com_result CreateInstance(const hypCLSID& clsid,
				  IUnknown* pUnknownOuter,
				  t_ulong ClsContext,
				  const hypIID& iid,
				  void** ppv)
{
	//Met le paramtre de sortie  0
	*ppv=0;

	IClassFactory* pIFactory=0;
	com_result hr=GetClassObject(clsid,
									ClsContext,
									NULL,
									IID_hypIClassFactory,
									(void**)(&pIFactory));

	if(hr==com_ok) {
		//cr le composant
		hr=pIFactory->CreateInstance(pUnknownOuter,iid,ppv);
		//libre la factory
		pIFactory->Release();
	}
	return hr;
}


/*!
\ingroup Component
Frees the unused component servers. If a server is unused, \c hypDllCanUnloadNow
must return a succeeded result code.
*/
hyp_ker_DLL void FreeUnusedLibraries(void)
{
	t_ContDllLoaded dll_unloaded;
	for(t_ContDllLoaded::const_iterator itor=gs_DllLoaded.begin();
	itor!=gs_DllLoaded.end();
	itor++) {
		try {
			CDllUtility dll;
			PTR_HYPDLLCANUNLOADNOW CanUnloadNow=static_cast<PTR_HYPDLLCANUNLOADNOW>(
				dll.GetFunctionAddress(*itor,g_NAME_HYPDLLCANUNLOADNOW));

			if( Succeeded(CanUnloadNow()) ) {
				dll.UnloadLibrary(*itor);
				dll_unloaded.insert(*itor);
			}			
		} catch(CException&) {
			//on teste la prochaine library
		}
	}

	//on efface les libraries dcharges
	for(itor=dll_unloaded.begin();
	itor!=dll_unloaded.end();
	itor++) {
		gs_DllLoaded.erase(*itor);
	}
}



/*!
\ingroup Component
Displays the loaded servers.
\param Stream Output stream used to display the list of loaded servers.
*/
hyp_ker_DLL void GetLoadedLibraries(std::ostream& Stream)
{
	Stream<<"->Loaded Component Servers\n";
	for(t_ContDllLoaded::const_iterator itor=gs_DllLoaded.begin();
	itor!=gs_DllLoaded.end();
	itor++) {
		Stream<<*itor<<'\n';
	}
	Stream<<"<-\n";
}


hyp_NAMESPACE_END_HYPKERNEL