/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Creation 10/05/01 PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL



//----------------------------------------------------------------
/*! \class CCmdInfoData
\ingroup CmdInfo
Defines the interface for data types in the command line:
- Command line - instanciated by CCmdInfo;
- Label enumerations - instanciated by CCmdInfoLabelEnum;
- Labels - instanciated by CCmdInfoLabel.
\internal
*/
//----------------------------------------------------------------

//----------------------------------------------------------------
/*! \class CCmdInfoArg
\ingroup CmdInfo
Stores the command line of arguments.
\internal
*/
//----------------------------------------------------------------
int CCmdInfoArg::m_Argc=0;
char** CCmdInfoArg::m_Argv=0;


//----------------------------------------------------------------
/*! \class CCmdInfoLabel
\ingroup CmdInfo
Defines a set of label
*/
//----------------------------------------------------------------
/*!
The correct construction of this object is done with CCmdInfo::GetValue function.
\param PosBegin Starting position of the label and its following parameters.
\param PosLast Ending position of the label.
\pre Static data, \c m_Argc and \c m_Argv have been initialized.
*/
CCmdInfoLabel::CCmdInfoLabel(int PosBegin,int PosLast) :
m_PosBegin(PosBegin),
m_PosLast(PosLast)
{

}

/*!
\return Number of parameters for the label.
*/
int CCmdInfoLabel::GetSize(void) const
{
	return m_PosLast-m_PosBegin-1;
}

/*!
\return Value of the specified parameter. Return an empty string if no parameter exits for the specified position.
\param Pos Position of the parameter related to the label. 0 corresponds to the first psoition.
*/
std::string CCmdInfoLabel::GetValue(int Pos) const
{
	if(Pos<GetSize())
		return m_Argv[m_PosBegin+Pos+1];
	else
		return std::string("");
}


//----------------------------------------------------------------
/*! \class CCmdInfoLabelEnum
\ingroup CmdInfo
Enumeration of labels. In a command line, several occurences are possible,
so to obtain the value of a label, the Hyperion Pattern introduce this intermediate
class.
*/
//----------------------------------------------------------------
/*!
Usualy created with a CCmdInfo object.
\param LabelName Name of the selected label.
\pre Static data, \c m_Argc and \c m_Argv have been initialized.
*/
CCmdInfoLabelEnum::CCmdInfoLabelEnum(const std::string& LabelName) :
m_LabelName(LabelName)
{

}

/*!
\return Number of labels in the command line.
*/
int CCmdInfoLabelEnum::GetSize(void) const
{
	return size();
}

/*!
\param Pos Position of the specified label. 0 coresponds to the first position.
\return Specified label. Returns an empty label if none exists.
*/
CCmdInfoLabel CCmdInfoLabelEnum::GetValue(int Pos) const
{
	if(Pos<GetSize())
		return at(Pos);
	else
		return CCmdInfoLabel();
}



//----------------------------------------------------------------
/*! \class CCmdInfo
\ingroup CmdInfo
Wrappes the command line of the application.
*/
//----------------------------------------------------------------
/*!
\param Argc Integer specifying how many arguments are passed to the program from the command line.
Because the program name is considered an argument, the integer is at least 1.
Transmitted via the \c main function.
\param Argv Array of null-terminated strings.
The first string ( \c Argv[0] ) is the program name, and each following string is an argument passed to the program from the command line.
The last pointer ( \c Argv[Argc] ) is \c NULL.
\param LabelKey Character which indicates the starting point of a label. 
*/
CCmdInfo::CCmdInfo(int Argc,char** Argv,char LabelKey) :
m_LabelKey(LabelKey)
{
	SetArg(Argc,Argv);
}

/*!
Gets the number of labels found in the command line.
*/
int CCmdInfo::GetSize() const
{
	int compt=0;
	int pos=1;
	while((pos=GetNextLabelPosition(pos))!=-1) {
		compt++;
		pos++;
	}
	return compt;
}

int CCmdInfo::GetNextLabelPosition(int Pos) const
{
	for(int i=Pos;i<m_Argc;i++) {
		if(m_Argv[i][0]==m_LabelKey) return i;
	}
	return -1;
}

int CCmdInfo::GetNextLabel(const std::string& LabelName,int Pos) const
{
	int pos=Pos;
	while((pos=GetNextLabelPosition(pos))!=-1) {
		if(GetLabelName(pos)==LabelName) return pos;
		pos++;
	}
	return -1;
}

std::string CCmdInfo::GetLabelName(int Pos) const
{
	if(Pos<m_Argc && m_Argv[Pos][0]==m_LabelKey) return &m_Argv[Pos][1];
	else return 0;
}

/*!
Gets the enumeration of a specified label.
\param RelativePos The position of the specified label in the command line.
*/
CCmdInfoLabelEnum CCmdInfo::GetValue(int RelativePos) const
{
	int compt=0;
	int pos=1;
	while( ((pos=GetNextLabelPosition(pos))!=-1) && (compt!=RelativePos) ) {
		compt++;
		pos++;
	}

	CCmdInfoLabelEnum LabelEnum(GetLabelName(pos));
		
	int pos_begin=GetNextLabel(LabelEnum.GetLabelName(),1);
	int pos_last=1;
	for(int i=0;i<GetSize();i++) {
		if( (pos_last=GetNextLabelPosition(pos_begin+1))==-1) pos_last=m_Argc;
		LabelEnum.push_back(CCmdInfoLabel(pos_begin,pos_last));
		pos_begin=pos_last;
		if( (pos_begin=GetNextLabel(LabelEnum.GetLabelName(),pos_begin))==-1) break;
	}	

	return LabelEnum;
}

/*!
\return The full path of the program. Equivalent to Argv[0].
*/
std::string CCmdInfo::GetModuleFileName() const
{
	return m_Argv[0];
}

/*!
\return The parameters formated in a string. The program name is omitted, use GetModuleFileName
to obtain the program name.
*/
std::string CCmdInfo::GetLineOfParameters() const
{
	std::string str_return;
	for(int i=1;i<m_Argc;i++) {
		str_return+=m_Argv[i];
		str_return+=" ";
	}
	return str_return;
}

/*!
\return The program name, the program path is omitted.
*/
std::string CCmdInfo::GetModuleName() const
{
	std::string full_path(m_Argv[0]);
	int pos=full_path.find_last_of('\\');
	return std::string(full_path,pos+1,std::string::npos);
}

/*!
\return The program path, the program name is omitted.
*/
std::string CCmdInfo::GetModulePath() const
{
	std::string full_path(m_Argv[0]);
	int pos=full_path.find_last_of('\\');
	return std::string(full_path,0,pos+1);
}

/*!
Inserts in the specified stream the formatted command line info.
*/
hyp_ker_DLL std::ostream& operator<<(std::ostream& Stream,const CCmdInfo& CmdInfo)
{
	Stream<<"Command Line [Pre-Processing]\n";
	for(int i=0;i<CmdInfo.m_Argc;i++) {
		Stream<<i<<"->"<<CmdInfo.m_Argv[i]<<"\n";
	}

	Stream<<"Command Line [Post-Processing]\n";
	for(i=0;i<CmdInfo.GetSize();i++) {
		CCmdInfoLabelEnum label_enum(CmdInfo.GetValue(i));
		Stream<<"Label : "<<label_enum.GetLabelName()<<'\n';
		for(int j=0;j<label_enum.GetSize();j++) {
			CCmdInfoLabel label=label_enum.GetValue(j);
			Stream<<'\t'<<j<<'\n';
			for(int k=0;k<label.GetSize();k++) {
				Stream<<"\t\t"<<label.GetValue(k)<<'\n';
			}
		}
	}
	return Stream;
}

hyp_NAMESPACE_END_HYPKERNEL