/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL

inline bool _IsDirectory(DWORD Attr) {
	return ((Attr != 0xffffffff) && (Attr & FILE_ATTRIBUTE_DIRECTORY)) ? true:false;
}

//---------------------------------------------------------
/*! \class CDir
\ingroup System
Wrapper class for walking file system directories.
If an error occurs, a CException object is thrown.
*/
//---------------------------------------------------------


CDir::CDir()
#ifdef hyp_WIN32
{
	::memset(&m_FD,0,sizeof(m_FD));
	m_H=NULL;
}
#endif

CDir::~CDir()
#ifdef hyp_WIN32
{
	if (m_H) ::FindClose(m_H);
}
#endif 

/*!
\todo Check if there are not extra '\\'
*/
std::string CDir::GetFullName(const std::string& FileName)
{
	return m_Path+FileName;	
}

/*!
Get the current opened path.
*/
std::string CDir::GetPath() const
{
	return m_Path;
}


/*!
Check if a component is in the current directory.
\param ComponentName Name of the component
\param Dir 
- true if the searched component is a directory
- false if the searched component is a file
\return true if the component has been found
*/
bool CDir::IsInDir(const std::string& ComponentName,bool Dir)
{
	bool b_return=false;
	Rewind(ComponentName);
	if(Search()==ComponentName) {
		b_return=true;
		if(Dir) b_return=IsSubDir();
	}

	return b_return;
}

/*!
Read the search data structure to find the next pattern.
\return The next file or path which corresponds to the pattern.
*/
std::string CDir::Search()
#ifdef hyp_WIN32
{
	if(m_H==NULL) {
		m_H=::FindFirstFile(GetFullName(m_Pattern).c_str(),&m_FD);
		if (m_H==INVALID_HANDLE_VALUE) hyp_THROW(CException);
	} else {
		if (::FindNextFile(m_H,&m_FD)==0) hyp_THROW(CException);
	}
	return m_FD.cFileName;
}
#endif


/*!
Open a path.
\note Use this member function before any operation with files.
\param PathName Name of the path that will be open.
\param Create True ; the path is created if it doesn't exist.
*/
void CDir::Open(const std::string& PathName,bool Create)
#ifdef hyp_WIN32
{
	if(PathName.length()>_MAX_DIR) hyp_THROW2( CException , ("Path name too long") );
	
	m_Path=PathName;
	if(*(m_Path.end()-1)!='\\') m_Path+='\\';
				
	DWORD attr;    
	attr=::GetFileAttributes(m_Path.c_str());
	if((attr == 0xffffffff) && Create ) {
		int pos=0;
		int pos_last=0;
		std::string subpath;
		bool bhard_disk=true;
		while(1) {
			pos=m_Path.find('\\',pos_last);
			if(pos==std::string::npos) break;
			pos++;	//pour prender '\'
			subpath+=m_Path.substr(pos_last,pos-pos_last);
			if( !bhard_disk ) {
				attr=::GetFileAttributes(subpath.c_str());
				if( !_IsDirectory(attr) )  
					if( ::CreateDirectory(subpath.c_str(),NULL) == FALSE )
						hyp_THROW2( CSysException , ("Path %s can not be created",subpath.c_str()) );
			} else {
				 bhard_disk=false;
			}
			pos_last=pos;
		}
	}
		
	attr=::GetFileAttributes(m_Path.c_str());
	if(!_IsDirectory(attr)) hyp_THROW2( CException , ("Path doesn't exist") );
}
#endif            




/*!
Rewind the search data structure
\param Pattern The pattern used to process the search.
*/
void CDir::Rewind(const std::string& Pattern)
#ifdef hyp_WIN32
{
    if (m_H) {
        ::FindClose(m_H);
        m_H = NULL;
    }
	m_Pattern=Pattern;
}
#endif


	
/*!
Checks if the current selection is a directory.
\return true if the current selection is a directory.
*/
bool CDir::IsSubDir()
#ifdef hyp_WIN32
{
    hyp_ASSERT( m_H );
    return _IsDirectory(m_FD.dwFileAttributes);
}
#endif

/*!
Creates a file
\todo Erase he file and create a empty file
\param FileName File name that will be created
*/
void CDir::CreateFile(const std::string& FileName) const
#ifdef hyp_WIN32
{
	HANDLE hnewfile=
		::CreateFile((m_Path+FileName).c_str(),  // pointer to name of the file
					0,			// access (read-write) mode
					0,          // share mode
					NULL,       // pointer to security attributes
					CREATE_ALWAYS,  // how to create
					FILE_ATTRIBUTE_NORMAL,  // file attributes
					NULL);      // handle to file with attributes to copy
	
	if(hnewfile==INVALID_HANDLE_VALUE) {
		hyp_THROW( CSysException );
	} else {
		::CloseHandle(hnewfile);
	}
}
#endif

// doxy_tag

/*!
Deletes a file.
\param FileName File name that will be deleted
*/
void CDir::DeleteFile(const std::string& FileName) const
#ifdef hyp_WIN32
{
	if(::DeleteFile((m_Path+FileName).c_str())==0) hyp_THROW( CSysException );
}
#endif

 

hyp_NAMESPACE_END_HYPKERNEL