/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL


//----------------------------------------------------------
/*! \class CDllUtility
\ingroup Dll
This classe wrappes the notion of dynamic library, so it is designed to be adapted
on a lot of platforms.
To be easily portable, each dll is identify by the standard string object.
But the primary mechanism is a specific platform data isolated by macros.
*/
//----------------------------------------------------------

CDllUtility::CDllContainer CDllUtility::ms_DllCont;


/*!
Load the library.
\param DllName Name of the loaded library
\param Init
- true: after the loading and before the unloading, the object called the exported function \href InitLibrary.
- false: The object doesn't call the InitLibrary function.
\warning Counting reference doesn't exist. If you call several time LoadLibrary,
one call to UnloadLibrary will unload physically the library.
*/
void CDllUtility::LoadLibrary(const std::string& DllName,bool Init)
#ifdef hyp_WIN32
{
	if(ms_DllCont.find(DllName)==ms_DllCont.end()) {
		HINSTANCE h=::LoadLibrary(DllName.c_str());
		if(h==NULL) hyp_THROW2( CSysException, ("%s",DllName.c_str()) );
		ms_DllCont[DllName]=CDllData(h,Init);
		if(Init) InitializeLibrary(DllName,dll_attach);		
	}	
}
#endif

/*!
Unload the library
\param DllName Name of the unloaded library.
\warning Any guarantee is given to unload the library if this library has been
loaded without the class.
*/
void CDllUtility::UnloadLibrary(const std::string& DllName)
#ifdef hyp_WIN32
{
	CDllContainer::iterator itor=ms_DllCont.find(DllName);

	if(itor==ms_DllCont.end()) hyp_THROW(CException);
	
	if(itor->second.Init()) InitializeLibrary(DllName,dll_detach);
	if(!itor->second.FreeLibrary()) hyp_THROW2( CSysException , ("%s",DllName.c_str()) );
	ms_DllCont.erase(itor);	
}
#endif

/*!
Unloads all the libraries that have been loaded by the class.
\throw CException Exception raised if an error occurs during one of the library unloading.
\warning Same warning as \ref unload_library.
*/
void CDllUtility::UnloadAllLibraries()
#ifdef hyp_WIN32
{
	t_DllCont cont_copy=ms_DllCont;
	bool error=false;

	for(t_DllCont::iterator itor=cont_copy.begin();itor!=cont_copy.end();itor++) {
		try {
			UnloadLibrary(itor->first);
		} catch(CException excep) {
			error=true;
		}
	}
	if(error) hyp_THROW( CException );

}
#endif

/*!
Gets the function adress of the library.
\param DllName Name of the library.
\param FunctionName Name of the function
\return Pointer to the function. Returns 0 if a problem occurs.
\warning The function must be exported. See .def file
*/
void* CDllUtility::GetFunctionAddress(const std::string& DllName,
							const std::string& FunctionName)
#ifdef hyp_WIN32
{
	CDllContainer::iterator itor=ms_DllCont.find(DllName);
	if(itor==ms_DllCont.end()) hyp_THROW( CException );
	void* p_function=::GetProcAddress(itor->second.GetH(),FunctionName.c_str());
	if(!p_function)	hyp_THROW2( CSysException , ("Unable to find adress of the function.") );
	return p_function;
}
#endif
	
	
/*!
Calls the InitLibrary function.
*/
void CDllUtility::InitializeLibrary(const std::string& DllName,EDllReason Reason)
{
	PTR_INITLIBRARY InitLibrary=
		static_cast<PTR_INITLIBRARY>(GetFunctionAddress(DllName,"InitLibrary"));
	
	if(InitLibrary!=0) {
		//construction du tableau des paramtres multiples
		//a completer suivant les besoins
		std::vector<std::string> _param;
		_param.push_back(DllName);	
		InitLibrary(_param,Reason);		
	}
}

/*!
Displays the list of the loaded libraries in the specified stream.
*/
hyp_ker_DLL std::ostream& operator<<(std::ostream& Stream,const CDllUtility& Dll)
{
	Stream<<"-->List of loaded dll\n"
		<<"size"<<Dll.ms_DllCont.size()<<'\n';

	for(CDllUtility::CDllContainer::const_iterator itor=Dll.ms_DllCont.begin();
	itor!=Dll.ms_DllCont.end();
	itor++)	{
		Stream<<itor->first<<" - h :"
			<<std::hex<<itor->second.GetH()
			<<" - init library :"<<std::boolalpha<<itor->second.Init()
			<<'\n';
	}

	Stream<<"<--"<<std::endl;
	return Stream;
}


//----------------------------------------------------------
//CDllData
//----------------------------------------------------------
CDllUtility::CDllData::CDllData(HINSTANCE H,bool InitLibrary) :
m_H(H),
m_InitLibrary(InitLibrary)
{

}


CDllUtility::CDllData::CDllData() :
m_H(0),
m_InitLibrary(false)
{

}

CDllUtility::CDllData::CDllData(const CDllData& Data) :
m_H(Data.m_H),
m_InitLibrary(Data.m_InitLibrary)
{
	
}


HINSTANCE& CDllUtility::CDllData::GetH()
{
	hyp_ASSERT(m_H!=NULL);
	return m_H;
}

/*!
\todo un static_cast<bool)(BOOL) customis...
*/
bool CDllUtility::CDllData::FreeLibrary()
{
	BOOL b=::FreeLibrary(GetH());
	m_H=NULL;
	return b==TRUE ? true : false;
}

const HINSTANCE& CDllUtility::CDllData::GetH() const
{
	hyp_ASSERT(m_H!=NULL);
	return m_H;
}

bool CDllUtility::CDllData::Init() const
{
	return m_InitLibrary;
}


//----------------------------------------------------------
//CDllContainer
//----------------------------------------------------------
/*!
\todo unload the libraries at the end of tha application
*/
CDllUtility::CDllContainer::~CDllContainer()
{
	//hyp_ASSERT(0);
	try {
		CDllUtility dll;
		dll.UnloadAllLibraries();
	} catch(CException excep) {
		hyp_TRACE( ("Error during the unloading of the remain dlls.") );
	}
}

hyp_NAMESPACE_END_HYPKERNEL
