/*
***************************************************************************************************
***************************************************************************************************
***
***    File Name: WFProfiler.h
***  Description: Header file to define the WFProfiler macros and function declarations. 
***        Notes: These macros are specific to the Intel Pentium processors. Also, the code
***               is specific to the MS Visual C++ compiler due to the __int64 declarations.
*** 
***               Unfortunately, these macros are useless to determine small operations such as
***               single shifts, single adds etc. For some reason, they reflect as taking -2 
***               clock cycles to complete. Is this due to caching? Multi-processing? Who knows.
***               When testing from Start()-Stop() with no instructions in between, I repeatedly
***               recieve a 0-cycle count as expected. Intel even states that the RDTSC instruction
***               is not perfect - especially when combined with CPUID. Check out the web-site:
***
***                     http://developer.intel.com/ for more information. (Search for RDTSC)...
***
***
***      Example:
***
***          #include <windows.h>
***          #include "..\Header Files\WFProfiler.h"
***
***
***          int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd )
***          {
***
***            WF_PROFILER profile;
***            int y=0;
***
***            Reset(profile);
***            Start(profile);
***
***            // This section is being timed...
***            for (int x=0; x<1000; x++)
***            {
***              y = rand() % 32;
***            }
***
***            Stop(profile);
***
***
***            // ****************************************************************
***            // ****************************************************************
***            // How long was the average pass through the FOR loop...?
***            // Exit Code tells ya (Mine shows 18-cycles - P3-700Mhz)...
***            return (int) ((int) Duration(profile) / x);  
***
***          }
***
***
***       Author: J.W. Heikkila, Jr.
***
***    Copyright:
***
***                 Confidential and Proprietary Source Code for
***                      Wild-Fire Gaming Systems, Inc.
***                     437 Jamesway, Marion, Ohio 43302
***
***    Revisions:
***
***              10/05/2000 - JWH - Original version.
***
***    Copyright (c) 2000, Wild-Fire Gaming Systems, Inc. All rights reserved.
***
***************************************************************************************************
***************************************************************************************************
*/

//hyp.ker.profiler.h
//inclusion de WfProfiler dans le kernel d'Hyperion 02/05/01 Tycho
//modification 02/05/01 Tycho


#ifndef _WFPROFILER_H_
#define _WFPROFILER_H_

#ifndef hypker_INSIDE
#error Inclusion not allowed (include hyp.ker.extern.h instead)
#endif

#ifdef hypker_SHOW_INCLUDES
#pragma message( "hypker include : " __FILE__ )
#endif



//******************************************************************************************
//******************************************************************************************
//
// Many compilers do not support the RDTSC and CPUID assembly commands. For those
// compilers, we need to force feed these commands into the code directly.
#define CPUID  __asm _emit 0x0F __asm _emit 0xA2 
#define RDTSC  __asm _emit 0x0F __asm _emit 0x31 
#define PUSHAD __asm _emit 0x60                  
#define POPAD  __asm _emit 0x61


//******************************************************************************************
//******************************************************************************************
//
// Starts the Clock Cycle Counter...
#define wf_START_NOW(pro) __asm                                                                     \
{                                    /* Actual Reason for Instruction:                        */ \
  __asm PUSHAD                       /* Save all registers before we begin...                 */ \
  __asm CPUID                        /* Tell the CPU to Serialize Instruction Execution       */ \
  __asm MOV [pro].Started, 1         /* Show we have started this profiler going              */ \
  __asm RDTSC                        /* Read the CPU Clock Cycle Count                        */ \
  __asm MOV [pro].StartHigh, EDX     /* Move the High DWORD into the Profile settings (start) */ \
  __asm MOV [pro].StartLow, EAX      /* Move the Low DWORD into the Profile settings (start)  */ \
  __asm POPAD                        /* Restore the registers the way they were               */ \
}



//******************************************************************************************
//******************************************************************************************
//
// Stops the Clock Cycle Counter...
#define wf_STOP_NOW(pro) __asm                                                                       \
{                                    /* Actual Reason for Instruction:                        */ \
  __asm PUSHAD                       /* Save all registers before we begin...                 */ \
  __asm CPUID                        /* Tell the CPU to Serialize Instruction Execution       */ \
  __asm MOV [pro].Started, 0         /* Show we have finished this profiler test              */ \
  __asm RDTSC                        /* Read the CPU Clock Cycle Count                        */ \
  __asm MOV [pro].EndHigh, EDX       /* Move the High DWORD into the Profile settings (end)   */ \
  __asm MOV [pro].EndLow, EAX        /* Move the Low DWORD into the Profile settings (end)    */ \
  __asm POPAD                        /* Restore the registers the way they were               */ \
}



//******************************************************************************************
//******************************************************************************************
//
//  In order to get an accurate reading, we have to run the StartNow()
//  four times. We will take a base time from the last time SartNow()
//  was executed until the StopNow() below is executed. This will give
//  us an idea of how long it takes to execute the instructions from  
//  the RDTSC in StartNow() until the  RDTSC in StopNow(). Store this 
//  as the Base Value and then we start the profiling...              
// 
#define wf_START(p) \
{                \
  wf_START_NOW(p);   \
  wf_STOP_NOW(p);    \
  ::hyp::hypker::SetDuration(p);\
  wf_START_NOW(p);   \
  wf_STOP_NOW(p);    \
  ::hyp::hypker::SetDuration(p);\
  wf_START_NOW(p);   \
  wf_STOP_NOW(p);    \
  ::hyp::hypker::SetDuration(p);\
  wf_START_NOW(p);   \
  wf_STOP_NOW(p);    \
  ::hyp::hypker::SetDuration(p);\
  wf_STOP_NOW(p);   \
}



//******************************************************************************************
//******************************************************************************************
//
// I think this is self-explanatory...
#define wf_STOP(p) wf_STOP_NOW(p);                  


struct WF_Profiler {
	unsigned Base;
	unsigned StartHigh;
	unsigned StartLow;
	unsigned EndHigh;
	unsigned EndLow;
	unsigned Started;
};

hyp_ker_DLL void Clear(WF_Profiler& data);
hyp_ker_DLL t_int64 Duration(const WF_Profiler& data);
hyp_ker_DLL void SetDuration(WF_Profiler& data);
hyp_ker_DLL void Reset(WF_Profiler& data);	





#endif  // _WFPROFILER_H_

