/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.kernel.core.h"


hyp_NAMESPACE_BEGIN_HYPKERNEL


//pour la key clsid
const char* g_CLSID_REGISTRY="clsid";
const char* g_PATH_INPROCESS_SUBKEY="path_inprocess";
const char* g_PROGID_SUBKEY="FriendlyName";

//pour la key ProgID
const char* g_PROGID_REGISTRY="progid";
const char* g_CLSID_SUBKEY="clsid";



//------------------------------------------------------------
/*! \class CRegistryUtility
\ingroup Registry
Utility class which permits to register easily the Hyperion Components.
This class could use several registries which implement the \ref IImplRegistry.
If the macro hyp_USE_REGISTRY...
*/
//------------------------------------------------------------

/*!
Indicates which registry system to use.
If \c hyp_USE_REGISTRY is defined, the class uses the Win32 registry by default.
*/
CRegistryUtility::CRegistryUtility(CRegistryUtility::ERegistryType Style)
{
	switch(Style) {
	case reg_win32:
		m_pImplRegistry=new CWin32ImplRegistry;
		break;		
	case reg_std:
		m_pImplRegistry=new CStdImplRegistry;
		break;
	default:
		m_pImplRegistry=0;
		hyp_THROW_COM(com_bad_arguments);
	}
}

CRegistryUtility::~CRegistryUtility()
{
	if(m_pImplRegistry) {
		delete m_pImplRegistry;
	}
}

/*!
Registers the component.
\todo Change the name of the functions (remove server)
\param DllName Full path of the Dll where the component is registered.
\param CLSID CLSID of the component.
\param FriendlyName Friendly name of the component.
*/
void CRegistryUtility::RegisterServer(const std::string& DllName,
											const CGuid& CLSID,
											const std::string& FriendlyName)
{
	//key clsid
	m_pImplRegistry->SetKey(g_CLSID_REGISTRY,CLSID.ToString());
	m_pImplRegistry->SetValueNameAndValue(g_CLSID_REGISTRY,
		CLSID.ToString(),
		g_PATH_INPROCESS_SUBKEY,
		DllName);
	m_pImplRegistry->SetValueNameAndValue(g_CLSID_REGISTRY,
		CLSID.ToString(),
		g_PROGID_SUBKEY,
		FriendlyName);

	//key progid
	//FIXTODO : creation automatique des repertoires clsid et proid?
	if(!FriendlyName.empty()) {
		m_pImplRegistry->SetKey(g_PROGID_REGISTRY,FriendlyName);
		m_pImplRegistry->SetValueNameAndValue(g_PROGID_REGISTRY,
			FriendlyName,
			g_CLSID_SUBKEY,
			CLSID.ToString());
	}
}

/*!
Unregisters the component.
\param CLSID CLSID of the component.
*/	
void CRegistryUtility::UnregisterServer(const CGuid& CLSID)
{
	//on cherche si il y a un friendly name
	std::string friendly_name=m_pImplRegistry->GetValueFromValueName(g_CLSID_REGISTRY,
		CLSID.ToString(),
		g_PROGID_SUBKEY);

	//mme si la chaine est vide, aucune exception leve
	m_pImplRegistry->DeleteKey(g_PROGID_REGISTRY,
			friendly_name);

	//on efface clsid
	m_pImplRegistry->DeleteKey(g_CLSID_REGISTRY,
		CLSID.ToString());
}

/*!
Get the full path of the server which is located the component.
\param CLSID CLSID of the component.
\return Path of the server.
*/
std::string CRegistryUtility::GetPathInprocessServer(const CGuid& CLSID)
{
	std::string str_return;
	str_return=m_pImplRegistry->GetValueFromValueName(g_CLSID_REGISTRY,
			CLSID.ToString(),
			g_PATH_INPROCESS_SUBKEY);

	return str_return;
}

/*!
Gets the CLSID of a component designated by a friendly name.
\param FriendlyName Frindly name of the component.
\return Founded CLSID of the component.
See \ref GetProgIDFromCLSID
*/
CGuid CRegistryUtility::GetCLSIDFromProgID(const std::string& FriendlyName)
{
	std::string clsid_string=m_pImplRegistry->GetValueFromValueName(g_PROGID_REGISTRY,
			FriendlyName,
			g_CLSID_SUBKEY);
	return CGuid(clsid_string);
}
/*!
Gets the friendly name of a component designated by a CLSID.
\param GUID GUID of the component.
\return Founded friendly name of the component.
See \ref GetCLSIDFromProgID
*/
std::string CRegistryUtility::GetProgIDFromCLSID(const CGuid& GUID)
{
	std::string prog_id_string=m_pImplRegistry->GetValueFromValueName(g_CLSID_REGISTRY,
			GUID.ToString(),
			g_PROGID_SUBKEY);
	return prog_id_string;
}


hyp_NAMESPACE_END_HYPKERNEL