/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Acknowledgements stx
 Creation 20/04/01 PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.kernel.core.h"

#ifdef hyp_WIN32
#include <sys/types.h>
#include <sys/timeb.h>
#include <time.h>
#endif

hyp_NAMESPACE_BEGIN_HYPKERNEL

//---------------------------------------------------------
/*! \class CTime
\ingroup System
Wrappes the notion of time, date and so on.
Based on the Sbasten Baron's implementation. Originlly created for the stx library, this class
has been updated with some minor functions to be integrated in the Hyperion Pattern framework
Definition World Time Standard:
- UTC (Universal Coordinated Time)
- GMT (Greenwich Mean Time)
\todo test systematique des ftime...
*/
//---------------------------------------------------------


/*!
Constructor by default. The duration is infinite by default.
*/
CTime::CTime()
{
	SetInfinite();
}

/*!
Constructor with seconds and nanoseconds.
*/
CTime::CTime( t_int64 sec, t_ulong nsec ) :
m_Sec(sec),
m_NanoSec(nsec)
{

}


/*!
Returns true if the date represents an infinite time.
*/
bool CTime::IsInfinite() const
{
	return m_NanoSec==0xFFFFFFFF;
}

/*!
Sets the date to an infinited time.
*/
void CTime::SetInfinite()
{
	m_Sec=0;
	m_NanoSec=0xFFFFFFFF;
}


/*!
Sets the date to the current data in the local time zone.
\todo summer time
*/
void CTime::SetCurrent()
#ifdef hyp_WIN32
{
	_timeb timebuffer;
	::_ftime( &timebuffer );
	m_Sec = timebuffer.time - ( 60L * timebuffer.timezone );
	m_NanoSec = timebuffer.millitm * 1000000L;
}
#endif

/*!
Sets the date to the GMT=0
*/
void CTime::SetCurrentGMT()
#ifdef hyp_WIN32
{
	_timeb timebuffer;
	::_ftime( &timebuffer );
	m_Sec=timebuffer.time;
	m_NanoSec=timebuffer.millitm*1000000L;
}
#endif

/*!
Sets the date to the local GMT time zone
*/
void CTime::SetCurrentLocalGMT()
#ifdef hyp_WIN32
{
	_timeb timebuffer;
	::_ftime( &timebuffer );
	m_Sec=timebuffer.time-(60L*timebuffer.timezone);
	m_NanoSec=timebuffer.millitm*1000000L;
}
#endif


/*!
Sets the date.
*/
void CTime::Set(
  t_ulong Year,
  t_ulong Month,
  t_ulong Day,
  t_ulong Hour,
  t_ulong Minute,
  t_ulong Second,
  t_ulong NanoSecond )
#ifdef hyp_WIN32
{
	tm t;
	t.tm_year=Year - 1900;
	t.tm_mon=Month - 1;
	t.tm_mday=Day;
	t.tm_hour=Hour;
	t.tm_min=Minute;
	t.tm_sec=Second;
	t.tm_isdst=-1;

	m_NanoSec=NanoSecond;

	if((m_Sec=::mktime(&t))==-1) {
		SetInfinite();
		hyp_THROW( CException );
	}	
}
#endif

/*! \name Access functions
Return the total number of X that is contained in the object.
X represents :
- nano-seconds
- seconds
- minutes
- hours
- days
- months
- years
*/	
//@{
t_ulong CTime::Year() const
{
	time_t s=m_Sec;
	tm *t=::gmtime( &s );
	return t->tm_year + 1900;
}

t_ulong CTime::Month() const
{
	time_t s = m_Sec;
	tm *t = gmtime( &s );
	return t->tm_mon + 1;
}

t_ulong CTime::Day() const
{
	time_t s=m_Sec;
	tm *t=::gmtime( &s );
	return t->tm_mday;
}

t_ulong CTime::Hour() const
{
	time_t s=m_Sec;
	tm *t=::gmtime( &s );
	return t->tm_hour;
}

t_ulong CTime::Minute() const
{
	time_t s=m_Sec;
	tm *t=::gmtime( &s );
	return t->tm_min;
}

t_ulong CTime::Second() const
{
	time_t s=m_Sec;
	tm *t=::gmtime( &s );
	return t->tm_sec;
}

t_ulong CTime::NanoSecond() const
{
	return m_NanoSec;
}
//@}
  
CTime& CTime::operator-=(const CTime& Time)
{
	if(Time.IsInfinite()) {
		SetInfinite();
	} else {
		m_Sec -=Time.m_Sec;
		if(m_NanoSec>=Time.m_NanoSec) {
			m_NanoSec -=Time.m_NanoSec;
		} else {
			m_NanoSec=0xFFFFFFFF-m_NanoSec;
			m_NanoSec+=Time.m_NanoSec;
			m_NanoSec++;
			m_Sec--;
		}
	}
	return *this;
}

CTime& CTime::operator+=(const CTime& Time)
{
	if(Time.IsInfinite()) {
		SetInfinite();
	} else {
		m_Sec+=Time.m_Sec;
		m_NanoSec+=Time.m_NanoSec;
		if(m_NanoSec>=1000000000UL) {
			// Gestion de la retenue
			m_NanoSec-=1000000000UL;
			m_Sec++;
		}
	}
	return *this;
}

bool CTime::operator==(const CTime& Time) const
{
	if(Time.IsInfinite()) {
		if(IsInfinite()) return true;
		else return false;
	}
	return ( (m_Sec==Time.m_Sec) && (m_NanoSec==Time.m_NanoSec) );
}

bool CTime::operator!=(const CTime& Time) const
{
	return !(Time==*this);
}

/*!
Formats the date to a string.
\todo porte global des operateurs de flux
*/
hyp_ker_DLL std::ostream& operator<<(std::ostream& Stream,const CTime& Time)
{
	Stream<<std::setfill('0')<<std::setw(2)<<Time.Day()<<"/"
		<<std::setw(2)<<Time.Month()<<"/"
		<<Time.Year()<<" - "
		<<std::setw(2)<<Time.Hour()<<":"
		<<std::setw(2)<<Time.Minute()<<":"
		<<std::setw(2)<<Time.Second();

	return Stream;
}

hyp_NAMESPACE_END_HYPKERNEL