/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.kernel.h"
#include"hyp.kernel.std.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL

//----------------------------------------------------------
/*! \class CFactory
\ingroup StdComponent
Implements the Factory used by the component server. So this  class can
manage the creation of all the components located in the same server.
*/
//----------------------------------------------------------


long CFactory::ms_cServerLocks=0;
std::string CFactory::ms_NameModule;
CFactory::ContFactoryData CFactory::ms_FactoryData;


CFactory::CFactory(const CFactoryData* pData)
: m_cRef(1)
{
	hypstd_TRACE( ("CFactory : \tAppel de CFactory()") );
	m_pData=pData;
}

CFactory::~CFactory()
{

}


com_result CFactory::QueryInterface(const hypIID& iid, void** ppv)
{ 	
	IUnknown* pI ;
	if ((iid == IID_hypIUnknown) || (iid == IID_hypIClassFactory))
	{
		pI= this ;
	}
	else
	{
		*ppv = 0 ;
		return com_no_interface;
	}
	pI->AddRef() ;
	*ppv = pI ;
	return com_ok;
}


t_ulong CFactory::AddRef()
{
	hypstd_TRACE( ("CFactory : Appel de AddRef()") );
	return ++m_cRef;
}

t_ulong CFactory::Release()
{
	hypstd_TRACE( ("CFactory : Appel de Release()") );
	if (--m_cRef == 0)
	{
		delete this ;
		return 0 ;
	}
	return m_cRef ;
}


//Implementation de  IClassFactory

com_result CFactory::CreateInstance(IUnknown* pUnknownOuter,
                                           const hypIID& iid,
                                           void** ppv)
{
	hypstd_TRACE( ("CFactory :\t Appel de CreateInstance.") );
		
	// Aggregate only if the requested IID is IID_IUnknown.
	if ((pUnknownOuter != 0) && (iid != IID_hypIUnknown))
	{
		return com_no_aggregation;
	}

	// Create the component.
	CUnknown* pNewComponent ;
	com_result hr = m_pData->CreateInstance(pUnknownOuter,
	                                     &pNewComponent) ;
	if (hr!=com_ok)
	{
		return hr ;
	}

	// Initialize the component.
	hr = pNewComponent->Init() ;
	if (hr!=com_ok)
	{
		// Initialization failed.  Release the component.
		pNewComponent->NondelegatingRelease() ;
		return hr ;
	}

	// Get the requested interface.
	hr = pNewComponent->NondelegatingQueryInterface(iid, ppv) ;

	// Release the reference held by the class factory.
	pNewComponent->NondelegatingRelease() ;
	return hr ;
}


com_result CFactory::LockServer(bool bLock)
{
	if (bLock) 
	{
		++ms_cServerLocks;
	}
	else
	{
		--ms_cServerLocks;
	}
	return com_ok;
}

/*!
Helps to implement hypDllGetClassObject
*/
com_result CFactory::GetClassObject(const hypCLSID& clsid,
										   const hypIID& iid,
										   void** ppv)
{
	hypstd_TRACE( ("CFactory : \t Appel de GetClassObject.") );
	
	if ((iid != IID_hypIUnknown) && (iid != IID_hypIClassFactory))
	{
		return com_no_interface;
	}

	// Traverse the array of data looking for this class ID.
	for (ContFactoryDataConstItor itor=ms_FactoryData.begin();
		itor!=ms_FactoryData.end();itor++)
	{
		const CFactoryData* pData = itor;
		hyp_ASSERT(pData);
		if (pData->IsClassID(clsid))
		{
			// Found the ClassID in the array of components we can
			// create.  So create a class factory for this component.
			// Pass the CFactoryData structure to the class factory
			// so that it knows what kind of components to create.
			
			//FIXTODO : desactiver le lancement d'exception
			*ppv = static_cast<IUnknown*>(new CFactory(pData));
			if (*ppv == 0)
			{
				return com_out_of_memory;
			}
			return com_ok;
		}
	}
	return com_class_not_available;
}

bool CFactory::IsLocked()
{
	return (ms_cServerLocks > 0);
}

/*!
Helps to implement hypDllRegisterAll.
*/
com_result CFactory::RegisterAll()
{
	com_result hr=com_ok;
		
	for (ContFactoryDataConstItor itor=ms_FactoryData.begin();
		itor!=ms_FactoryData.end();itor++) {
		try{
			CRegistryUtility reg_util;
			reg_util.RegisterServer(ms_NameModule,*(itor->m_pCLSID),itor->m_ComponentName);
		} catch(CException&) {
			hr=com_unexpected;
		}
	}
	
	return hr;
}

/*!
Helps to implement hypDllUnregisterAll.
*/
com_result CFactory::UnregisterAll()
{
	com_result hr=com_ok;
		
	for (ContFactoryDataConstItor itor=ms_FactoryData.begin();
		itor!=ms_FactoryData.end();itor++) 	{
		try{
			CRegistryUtility reg_util;
			reg_util.UnregisterServer(*(itor->m_pCLSID));
		} catch(CException&){
			hr=com_unexpected;
		}
	}
	return hr;
}

	
/*!
Helps to implement hypDllCanUnloadNow
Checks if the factory is free or no, so if the component server can be unloaded.
*/
com_result CFactory::CanUnloadNow()
{
	if (CUnknown::ActiveComponents() || IsLocked())	{
		return com_false;
	} else {
		return com_ok;
	}
}

/*
Initializes  the static data members of the CFactory class.
\param pFactory Data Pointer to an C style array of CFactoryData objects.
\param Size Size of the array, so equal to the number of components that are hosted by the server.
\param NameModule Full path of the server.
*/
void CFactory::InitStaticMembers(const CFactoryData* pFactoryData,
								  int Size,
								  const std::string& NameModule)
{
	ms_FactoryData.clear();
	for(int i=0;i<Size;i++) {
		ms_FactoryData.push_back(pFactoryData[i]);
	}

	CFactory::ms_NameModule=NameModule;
}


hyp_NAMESPACE_END_HYPKERNEL